@@ -7,7 +7,7 @@
 #' well as raster predictions for each model when raster data is input. The evaluation statistics in the 
 #' results table should aid users in identifying model settings that balance fit and predictive ability. See
 #' the extensive vignette for fully worked examples: 
-#' <https://jamiemkass.github.io/ENMeval/articles/ENMeval-2.0.0-vignette.html>.
+#' <https://jamiemkass.github.io/ENMeval/articles/ENMeval-2.0-vignette.html>.
 #' 
 #' @param occs matrix / data frame: occurrence records with two columns for longitude and latitude 
 #' of occurrence localities, in that order. If specifying predictor variable values
@@ -33,13 +33,18 @@
 #' See Details and ?partitions for descriptions of these settings.
 #' @param other.settings named list: used to specify extra settings for the analysis. 
 #' All of these settings have internal defaults, so if they are not specified the analysis will be run 
-#' with default settings. See Details for descriptions of these settings.
+#' with default settings. See Details for descriptions of these settings, including how to specify arguments
+#' for maxent.jar.
 #' @param categoricals character vector: name or names of categorical environmental variables. If not specified,
 #' all predictor variables will be treated as continuous unless they are factors. If categorical variables
 #' are already factors, specifying names of such variables in this argument is not needed.
 #' @param doClamp boolean: if TRUE (default), model prediction extrapolations will be restricted to the upper and lower
 #' bounds of the predictor variables. Clamping avoids extreme predictions for environment values outside
-#' the range of the training data. If extrapolation is a study aim, this should be set to FALSE.
+#' the range of the training data. If free extrapolation is a study aim, this should be set to FALSE, but
+#' for most applications leaving this at the default of TRUE is advisable to avoid unrealistic predictions. 
+#' When predictor variables are input, they are clamped internally before making model predictions when clamping is on.
+#' When no predictor variables are input and data frames of variable values are used instead (SWD format),
+#' validation data is clamped before making model predictions when clamping is on.
 #' @param clamp.directions named list: specifies the direction ("left" for minimum, "right" for maximum) 
 #' of clamping for predictor variables -- (e.g., \code{list(left = c("bio1","bio5"), right = c("bio10","bio15"))}).
 #' @param user.enm ENMdetails object: a custom ENMdetails object used to build models. 
@@ -113,7 +118,8 @@
 #' spatial partitions only) to calculate each with respect to the partitioned background only 
 #' (i.e., training occurrences are compared to training background, and validation occurrences 
 #' compared to validation background).\cr*
-#' other.args - named list: any additional model arguments not specified for tuning.\cr
+#' other.args - named list: any additional model arguments not specified for tuning; this can
+#' include arguments for maxent.jar, which are described in the software's Help file.\cr
 #' 
 #' For partition.settings, the current options are:\cr*
 #' orientation - character: one of "lat_lon" (default), "lon_lat", "lat_lat", or "lon_lon" (required for block partition).\cr* 
@@ -154,7 +160,8 @@
 #' 
 #' Warren, D. L., Glor, R. E., Turelli, M. & Funk, D. (2008) Environmental niche equivalency versus conservatism: quantitative approaches to niche evolution. \emph{Evolution}, \bold{62}: 2868-2883. \doi{10.1111/j.1558-5646.2008.00482.x}
 #' 
-#' @return An ENMevaluation object. See ?ENMevaluation for details.
+#' @return An ENMevaluation object. See ?ENMevaluation for details and description of the columns
+#' in the results table.
 #'
 #' @importFrom magrittr %>%
 #' @importFrom foreach %dopar%
@@ -175,13 +182,79 @@
 #' names(bg) <- names(occs)
 #' bg.z <- cbind(bg, raster::extract(envs, bg))
 #' bg.z$biome <- factor(bg.z$biome)
-#' tune.args <- list(fc = c("L","LQ","LQH","H"), rm = 1:5)
-#' os <- list(abs.auc.diff = FALSE, pred.type = "logistic", validation.bg = "partition")
+#' 
+#' # set other.settings -- pred.type is only for Maxent models
+#' os <- list(abs.auc.diff = FALSE, pred.type = "cloglog", validation.bg = "partition")
+#' # set partition.settings -- here's an example for the block method
+#' # see Details for the required settings for other partition methods
 #' ps <- list(orientation = "lat_lat")
 #' 
-#' e <- ENMevaluate(occs, envs, bg, tune.args = tune.args, partitions = "block", 
-#' other.settings = os, partition.settings = ps,
+#' # here's a run with maxnet -- note the tune.args for feature classes (fc)
+#' # and regularization multipliers (rm), as well as the designation of the
+#' # categorical variable we are using (this can be a vector if multiple
+#' # categorical variables are used)
+#' e.maxnet <- ENMevaluate(occs, envs, bg, 
+#' tune.args = list(fc = c("L","LQ","LQH","H"), rm = 1:5), 
+#' partitions = "block", other.settings = os, partition.settings = ps,
 #' algorithm = "maxnet", categoricals = "biome", overlap = TRUE)
+#' 
+#' # print the tuning results
+#' eval.results(e.maxnet)
+#' 
+#' # there is currently no native function to make raster model predictions for
+#' # maxnet models, but ENMeval can be used to make them like this:
+#' # here's an example where we make a prediction based on the L2 model
+#' # (feature class: Linear, regularization multiplier: 2) for our envs data
+#' mods.maxnet <- eval.models(e.maxnet)
+#' pred.L2 <- enm.maxnet@predict(mods.maxnet$fc.L_rm.2, envs, os)
+#' raster::plot(pred.L2)
+#' 
+#' #' # here's a run with maxent.jar -- note that if the R package rJava cannot 
+#' install or load, or if other issues with Java exist on your computer, 
+#' maxent.jar will not function
+#' e.maxnet <- ENMevaluate(occs, envs, bg, 
+#' tune.args = list(fc = c("L","LQ","LQH","H"), rm = 1:5), 
+#' partitions = "block", other.settings = os, partition.settings = ps,
+#' algorithm = "maxent.jar", categoricals = "biome", overlap = TRUE)
+#' 
+#' # print the tuning results
+#' eval.results(e.maxent.jar)
+#' # raster predictions can be made for maxent.jar models with dismo or ENMeval
+#' mods.maxent.jar <- eval.models(e.maxent.jar)
+#' pred.L2 <- dismo::predict(mods.maxent.jar$fc.L_rm.2, envs, args = "outputform=cloglog")
+#' pred.L2 <- enm.maxent.jar@predict(mods.maxent.jar$fc.L_rm.2, envs, os)
+#' raster::plot(pred.L2)
+#' 
+#' # this will give you the percent contribution (not deterministic) and
+#' # permutation importance (deterministic) values of variable importance for
+#' # Maxent models, and it only works with maxent.jar
+#' eval.variable.importance(e.maxent.jar)
+#' 
+#' # here's a run with BIOCLIM -- note that 1) we need to remove the categorical
+#' # variable here because this algorithm only takes continuous variables, and
+#' # that 2) the way BIOCLIM makes predicted is getting tuned (as opposed to the
+#' way the model is fit like maxnet or maxent.jar), namely, the tails of the 
+#' # distribution that are ignored when predicting (see ?dismo::bioclim)
+# e.bioclim <- ENMevaluate(occs, envs[[-9]], bg,
+# tune.args = list(tails = c("low", "high", "both")),
+# partitions = "block", other.settings = os, partition.settings = ps,
+# algorithm = "bioclim", overlap = TRUE)
+#' 
+#' # print the tuning results
+#' eval.results(e.bioclim)
+#' # make raster predictions with dismo or ENMeval
+#' mods.bioclim <- eval.models(e.bioclim)
+#' # note: the models for low, high, and both are actually all the same, and
+#' # the only difference for tuning is how they are predicted during
+#' # cross-validation
+#' pred.both <- dismo::predict(mods.bioclim$tails.both, envs, tails = "both")
+#' os <- c(os, list(tails = "both"))
+#' pred.both <- enm.bioclim@predict(mods.bioclim$tails.both, envs, os)
+#' raster::plot(pred.both)
+#' 
+#' # please see the vignette for more examples of model tuning, 
+#' # partitioning, plotting functions, and null models
+#' # https://jamiemkass.github.io/ENMeval/articles/ENMeval-2.0-vignette.html
 #' }
 #' 
 #' @export 
@@ -201,7 +274,7 @@ ENMevaluate <- function(occs, envs = NULL, bg = NULL, tune.args = NULL, partitio
   # legacy argument handling so ENMevaluate doesn't break for older code
   all.legacy <- list(occ, env, bg.coords, RMvalues, fc, occ.grp, bg.grp, method, bin.output, rasterPreds)
   if(sum(sapply(all.legacy, function(x) !is.null(x))) > 0) {
-    if(quiet != TRUE) message("* Running ENMeval v2.0.3 with legacy arguments. These will be phased out in the next version.")
+    if(quiet != TRUE) message("* Running ENMeval v2.0.4 with legacy arguments. These will be phased out in the next version.")
   }
   if(!is.null(occ)) occs <- occ
   if(!is.null(env)) envs <- env
@@ -249,14 +322,14 @@ ENMevaluate <- function(occs, envs = NULL, bg = NULL, tune.args = NULL, partitio
   if(is.null(partition.settings)) partition.settings <- list(orientation = "lat_lon", 
                                                              aggregation.factor = 2, 
                                                              kfolds = 5)
-  if(is.null(other.settings)) other.settings <- list(abs.auc.diff = TRUE, 
+  other.settings <- c(other.settings, list(abs.auc.diff = TRUE, 
                                                      pred.type = "cloglog", 
-                                                     validation.bg = "full")
+                                                     validation.bg = "full"))
   # add whether to use ecospat to other.settings to avoid multiple calls to require()
   other.settings <- c(other.settings, ecospat.use = ecospat.use)
   
   # make sure taxon name column is not included
-  if(class(occs[,1]) == "character" | class(bg[,1]) == "character") stop("* If first column of input occurrence or background data is the taxon name, remove it and instead include the 'taxon.name' argument. The first two columns must be the longitude and latitude of the occurrence/background localities.")
+  if(inherits(occs[,1], "character") | inherits(bg[,1], "character")) stop("* If first column of input occurrence or background data is the taxon name, remove it and instead include the 'taxon.name' argument. The first two columns must be the longitude and latitude of the occurrence/background localities.")
   
   if(is.null(taxon.name)) {
     if(quiet != TRUE) message(paste0("*** Running initial checks... ***\n"))
@@ -343,6 +416,7 @@ ENMevaluate <- function(occs, envs = NULL, bg = NULL, tune.args = NULL, partitio
   # algorithm-specific errors
   enm@errors(occs, envs, bg, tune.args, partitions, algorithm, partition.settings, other.settings, 
              categoricals, doClamp, clamp.directions)
+
   
   ########################################################### #
   # ASSEMBLE COORDINATES AND ENVIRONMENTAL VARIABLE VALUES ####
@@ -465,29 +539,40 @@ ENMevaluate <- function(occs, envs = NULL, bg = NULL, tune.args = NULL, partitio
   # CLAMPING ####
   ################# #
   if(doClamp == TRUE) {
-    # record in other.settings
-    other.settings$doClamp <- TRUE
-    
+    # when predictor variable rasters are input
     if(!is.null(envs)) {
+      # assign both clamp directions to all variables if none are set
       if(is.null(clamp.directions)) {
         clamp.directions$left <- names(envs)
         clamp.directions$right <- names(envs)
       }
-      # record in other.settings
+      # record clamp directions in other.settings
       other.settings$clamp.directions <- clamp.directions
+      # run function to clamp predictor variable rasters
       envs <- clamp.vars(orig.vals = envs, ref.vals = rbind(occs.z, bg.z), 
-                         left = clamp.directions$left, right = clamp.directions$right, 
+                         left = clamp.directions$left, 
+                         right = clamp.directions$right, 
                          categoricals = categoricals)
       if(quiet != TRUE) message("* Clamping predictor variable rasters...")
     }else{
+      # if no predictor variable rasters are input, assign both clamp directions
+      # to all variable names (columns besides the first two, which should be
+      # coordinates) if none are set
+      # during cross-validation, validation data will be clamped using the
+      # clamp.vars() function
       if(is.null(clamp.directions)) {
         clamp.directions$left <- names(d[, 3:(ncol(d)-1)])
         clamp.directions$right <- names(d[, 3:(ncol(d)-1)])
       }
     }
-  }else{
-    other.settings$doClamp <- FALSE
   }
+  # record clamping choice as FALSE in other.settings regardless of doClamp
+  # selection -- this is because the clamping is done on the predictor rasters 
+  # or values directly, so that internally all model predictions are made with 
+  # clamping off
+  # when enm.predict() functions are run externally, users can specify
+  # other.settings$doClamp to turn on clamping functionality
+  other.settings$doClamp <- FALSE
   
   ###################### #
   # ASSIGN PARTITIONS ####
@@ -540,9 +625,9 @@ ENMevaluate <- function(occs, envs = NULL, bg = NULL, tune.args = NULL, partitio
   ################# #
   # print model-specific message
   if(is.null(taxon.name)) {
-    if(quiet != TRUE) message(paste("\n*** Running ENMeval v2.0.3 with", enm@msgs(tune.args, other.settings), "***\n"))
+    if(quiet != TRUE) message(paste("\n*** Running ENMeval v2.0.4 with", enm@msgs(tune.args, other.settings), "***\n"))
   }else{
-    if(quiet != TRUE) message(paste("\n*** Running ENMeval v2.0.3 for", taxon.name, "with", enm@msgs(tune.args, other.settings), "***\n"))
+    if(quiet != TRUE) message(paste("\n*** Running ENMeval v2.0.4 for", taxon.name, "with", enm@msgs(tune.args, other.settings), "***\n"))
   }
   
   ################# #
@@ -557,11 +642,15 @@ ENMevaluate <- function(occs, envs = NULL, bg = NULL, tune.args = NULL, partitio
   if(nrow(tune.tbl) == 0) tune.tbl <- NULL
   
   if(parallel) {
-    results <- tune.parallel(d, envs, enm, partitions, tune.tbl, other.settings, partition.settings, 
-                             user.val.grps, occs.testing.z, numCores, parallelType, user.eval, quiet)  
+    results <- tune.parallel(d, envs, enm, partitions, tune.tbl, doClamp, 
+                             other.settings, partition.settings, user.val.grps, 
+                             occs.testing.z, numCores, parallelType, user.eval, 
+                             algorithm, quiet)  
   }else{
-    results <- tune.regular(d, envs, enm, partitions, tune.tbl, other.settings, partition.settings,
-                            user.val.grps, occs.testing.z, updateProgress, user.eval, quiet)
+    results <- tune.regular(d, envs, enm, partitions, tune.tbl, doClamp, 
+                            other.settings, partition.settings, user.val.grps, 
+                            occs.testing.z, updateProgress, user.eval, 
+                            algorithm, quiet)
   }
   
   ##################### #
@@ -667,11 +756,11 @@ ENMevaluate <- function(occs, envs = NULL, bg = NULL, tune.args = NULL, partitio
     aic.settings <- other.settings
     aic.settings$pred.type <- pred.type.raw
     if(!is.null(envs)) {
-      pred.all.raw <- raster::stack(lapply(mod.full.all, enm@predict, envs, NULL, aic.settings))
+      pred.all.raw <- raster::stack(lapply(mod.full.all, enm@predict, envs, aic.settings))
     }else{
       pred.all.raw <- NULL
     }
-    occs.pred.raw <- dplyr::bind_rows(lapply(mod.full.all, enm@predict, occs[,-c(1,2)], NULL, aic.settings))
+    occs.pred.raw <- dplyr::bind_rows(lapply(mod.full.all, enm@predict, occs[,-c(1,2)], aic.settings))
     aic <- aic.maxent(occs.pred.raw, ncoefs, pred.all.raw)
     eval.stats <- dplyr::bind_cols(eval.stats, aic)
   }
@@ -689,13 +778,16 @@ ENMevaluate <- function(occs, envs = NULL, bg = NULL, tune.args = NULL, partitio
   if(is.null(clamp.directions)) clamp.directions <- list()
   
   # get variable importance for all models
-  varimp.all <- lapply(mod.full.all, enm@varimp)
+  variable.importance.all <- lapply(mod.full.all, enm@variable.importance)
+  
+  # remove the doClamp = FALSE recorded in other.settings to avoid confusion
+  other.settings$doClamp <- NULL
   
   # assemble the ENMevaluation object
   e <- ENMevaluation(algorithm = enm@name, tune.settings = as.data.frame(tune.tbl),
                      results = as.data.frame(eval.stats), results.partitions = val.stats.all,
                      predictions = mod.full.pred.all, models = mod.full.all, 
-                     variable.importance = varimp.all,
+                     variable.importance = variable.importance.all,
                      partition.method = partitions, partition.settings = partition.settings,
                      other.settings = other.settings, doClamp = doClamp, clamp.directions = clamp.directions, 
                      taxon.name = as.character(taxon.name),
