@@ -32,8 +32,8 @@ NULL
 #' @rdname tune.enm
 tune.train <- function(enm, occs.z, bg.z, mod.full, envs, tune.tbl.i, other.settings, partitions, quiet) {
   # get model predictions for training data
-  occs.pred <- enm@predict(mod.full, occs.z, tune.tbl.i, other.settings)
-  bg.pred <- enm@predict(mod.full, bg.z, tune.tbl.i, other.settings)
+  occs.pred <- enm@predict(mod.full, occs.z, other.settings)
+  bg.pred <- enm@predict(mod.full, bg.z, other.settings)
   
   # training AUC
   e <- dismo::evaluate(occs.pred, bg.pred)
@@ -59,12 +59,12 @@ tune.train <- function(enm, occs.z, bg.z, mod.full, envs, tune.tbl.i, other.sett
 #' @rdname tune.enm
 tune.validate <- function(enm, occs.train.z, occs.val.z, bg.train.z, bg.val.z, mod.k, nk, tune.tbl.i, other.settings, partitions, user.eval, quiet) {
   # get model predictions for training and validation data for partition k
-  occs.train.pred <- enm@predict(mod.k, occs.train.z, tune.tbl.i, other.settings)
-  occs.val.pred <- enm@predict(mod.k, occs.val.z, tune.tbl.i, other.settings)
-  bg.train.pred <- enm@predict(mod.k, bg.train.z, tune.tbl.i, other.settings)
+  occs.train.pred <- enm@predict(mod.k, occs.train.z, other.settings)
+  occs.val.pred <- enm@predict(mod.k, occs.val.z, other.settings)
+  bg.train.pred <- enm@predict(mod.k, bg.train.z, other.settings)
   # if the background was partitioned, get predictions for this subset; if not, it will be NULL
   if(nrow(bg.val.z) > 0) {
-    bg.val.pred <- enm@predict(mod.k, bg.val.z, tune.tbl.i, other.settings)
+    bg.val.pred <- enm@predict(mod.k, bg.val.z, other.settings)
   }else{
     bg.val.pred <- NULL
     # if(quiet != TRUE) message("\nNOTE: Background points were not partitioned for this analysis, so model validation will proceed on the full background.")
@@ -143,7 +143,10 @@ tune.validate <- function(enm, occs.train.z, occs.val.z, bg.train.z, bg.val.z, m
 }
 
 #' @rdname tune.enm
-tune.parallel <- function(d, envs, enm, partitions, tune.tbl, other.settings, partition.settings, user.val.grps, occs.testing.z, numCores, parallelType, user.eval, quiet) {
+tune.parallel <- function(d, envs, enm, partitions, tune.tbl, doClamp, 
+                          other.settings, partition.settings, user.val.grps, 
+                          occs.testing.z, numCores, parallelType, user.eval, 
+                          algorithm, quiet) {
   # set up parallel processing functionality
   allCores <- parallel::detectCores()
   if (is.null(numCores)) {
@@ -166,7 +169,9 @@ tune.parallel <- function(d, envs, enm, partitions, tune.tbl, other.settings, pa
   if(quiet != TRUE) message(paste0("Running in parallel using ", parallelType, "..."))
   
   results <- foreach::foreach(i = 1:n, .options.snow = opts, .export = "cv.enm") %dopar% {
-    cv.enm(d, envs, enm, partitions, tune.tbl[i,], other.settings, partition.settings, user.val.grps, occs.testing.z, user.eval, quiet)
+    cv.enm(d, envs, enm, partitions, tune.tbl[i,], doClamp, other.settings, 
+           partition.settings, user.val.grps, occs.testing.z, user.eval, 
+           algorithm, quiet)
   }
   if(quiet != TRUE) close(pb)
   parallel::stopCluster(cl)
@@ -174,7 +179,10 @@ tune.parallel <- function(d, envs, enm, partitions, tune.tbl, other.settings, pa
 }
 
 #' @rdname tune.enm
-tune.regular <- function(d, envs, enm, partitions, tune.tbl, other.settings, partition.settings, user.val.grps, occs.testing.z, updateProgress, user.eval, quiet) {
+tune.regular <- function(d, envs, enm, partitions, tune.tbl, doClamp, 
+                         other.settings, partition.settings, user.val.grps, 
+                         occs.testing.z, updateProgress, user.eval, algorithm, 
+                         quiet) {
   results <- list()
   n <- ifelse(!is.null(tune.tbl), nrow(tune.tbl), 1)
   
@@ -192,14 +200,18 @@ tune.regular <- function(d, envs, enm, partitions, tune.tbl, other.settings, par
     }
     # set the current tune settings
     tune.tbl.i <- tune.tbl[i,]
-    results[[i]] <- cv.enm(d, envs, enm, partitions, tune.tbl.i, other.settings, partition.settings, user.val.grps, occs.testing.z, user.eval, quiet)
+    results[[i]] <- cv.enm(d, envs, enm, partitions, tune.tbl.i, doClamp,
+                           other.settings, partition.settings, user.val.grps, 
+                           occs.testing.z, user.eval, algorithm, quiet)
   }
   if(quiet != TRUE) close(pb)
   return(results)
 }
 
 #' @rdname tune.enm
-cv.enm <- function(d, envs, enm, partitions, tune.tbl.i, other.settings, partition.settings, user.val.grps, occs.testing.z, user.eval, quiet) {
+cv.enm <- function(d, envs, enm, partitions, tune.tbl.i, doClamp, 
+                   other.settings, partition.settings, user.val.grps, 
+                   occs.testing.z, user.eval, algorithm, quiet) {
   envs.names <- names(d[, 3:(ncol(d)-2)])
   # unpack predictor variable values for occs and bg
   occs.xy <- d %>% dplyr::filter(pb == 1) %>% dplyr::select(1:2)
@@ -226,7 +238,12 @@ cv.enm <- function(d, envs, enm, partitions, tune.tbl.i, other.settings, partiti
     pred.envs <- d %>% dplyr::select(dplyr::all_of(envs.names))
   }
   
-  mod.full.pred <- enm@predict(mod.full, pred.envs, tune.tbl.i, other.settings)
+  # as bioclim can be tuned with different "tails" settings that affect not the 
+  # model but the prediction, these settings need to be moved from tune.tbl.i
+  # to other.settings as eval.predict() does not accept tune.args
+  if(algorithm == "bioclim") other.settings$tails <- tune.tbl.i$tails
+  
+  mod.full.pred <- enm@predict(mod.full, pred.envs, other.settings)
   # get evaluation statistics for training data
   train <- tune.train(enm, occs.z, bg.z, mod.full, envs, tune.tbl.i, other.settings, partitions, quiet)
   # make training stats table
@@ -242,7 +259,7 @@ cv.enm <- function(d, envs, enm, partitions, tune.tbl.i, other.settings, partiti
   if(partitions == "testing") {
     bg.val.z <- data.frame()
     occs.testing.zEnvs <- occs.testing.z %>% dplyr::select(dplyr::all_of(envs.names))
-    if(other.settings$doClamp == TRUE) {
+    if(doClamp == TRUE) {
       occs.testing.zEnvs <- clamp.vars(orig.vals = occs.testing.zEnvs, ref.vals = rbind(occs.z, bg.z), 
                                        left = other.settings$clamp.directions$left, right = other.settings$clamp.directions$right, 
                                        categoricals = other.settings$categoricals)
@@ -271,7 +288,7 @@ cv.enm <- function(d, envs, enm, partitions, tune.tbl.i, other.settings, partiti
     
     # if doClamp is on, make sure that the validation data for each validation model is also clamped
     # this means for each partition, making sure no values in validation data are more extreme than those in training data
-    if(other.settings$doClamp == TRUE) {
+    if(doClamp == TRUE) {
       val.z <- clamp.vars(orig.vals = rbind(occs.val.z, bg.val.z), ref.vals = rbind(occs.train.z, bg.train.z), 
                           left = other.settings$clamp.directions$left, right = other.settings$clamp.directions$right, 
                           categoricals = other.settings$categoricals)
